/*
Copyright Ideaconsult (C) 2005-2007 
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

package chemCast.data;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;

import javax.help.UnsupportedOperationException;

import org.openscience.cdk.interfaces.IAtomContainer;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.PropertyTranslator;
import ambit.data.molecule.SortedPropertyList;
import chemCast.events.DatasetChangeEvent;

public class FilteredMoleculesFile implements IAtomContainersList, PropertyChangeListener {
    /**
     * 
     */
	protected IAtomContainersList file;
    private static final long serialVersionUID = 6159377185736627200L;
    protected IObjectFilter filter = null;
    protected ArrayList<Integer> filtered;
    protected int selectedIndex = -1;
    protected SortedPropertyList sortedProperties;
    
    public FilteredMoleculesFile(IAtomContainersList file,IObjectFilter filter) {
        super();
        this.file = file;
        if (file instanceof MoleculesFile)
        	((MoleculesFile)file).addPropertyChangeListener(this);
        filtered = new ArrayList<Integer>();
        setFilter(filter);
        sortedProperties = new SortedPropertyList("");
        //System.out.println("FilteredMoleculesFile "+ toString() + " " + getAtomContainerCount() );
    }
    public synchronized IObjectFilter getFilter() {
        return filter;
    }
    public synchronized void setFilter(IObjectFilter filter) {
        this.filter = filter;
        filtered.clear();
        
        if (filter instanceof MultiplePropertyFilters) {
        	ArrayList<IPropertyFilter> filters = ((MultiplePropertyFilters) filter).getFilters();
	        for (int i=0; i < file.getAtomContainerCount();i++) {
	        	boolean ok = true;
	    		for (int f=0; f < filters.size();f++) {
	    			Object property = filters.get(f).getTag();
	    			if (property != null) {
	    				Object value = file.getProperty(i,property);
	    				if (value == null) {ok = false; break;}
	    				if (!filters.get(f).acceptProperty(property, value)) { ok = false; break;};
	    			}	
	    		}
	    		if (ok) filtered.add(new Integer(i));
	        }	
        } else if (filter instanceof IPropertyFilter) {
        	IPropertyFilter pf = (IPropertyFilter) filter;
	        for (int i=0; i < file.getAtomContainerCount();i++) {
        		Object key = pf.getTag();
        		if (key != null) {
		            Object value = file.getProperty(i,key);
		            if ((value != null) && pf.acceptProperty(key,value))
		            	filtered.add(new Integer(i));
	        	}
	        }

        } else 	
	        for (int i=0; i < file.getAtomContainerCount();i++) {
                //System.out.println("setFilter");
	            IAtomContainer o = file.getAtomContainer(i);
                //filter.preprocess(o);
	            if (filter.accept(o)) 
	                filtered.add(new Integer(i));
	        }
    }
    
    public int getAtomContainerCount() {
        return filtered.size();
    }
    
    public Object first() {
        return getAtomContainer(0);
    }
    
    public Object last() {
        return getAtomContainer(filtered.size()-1);
    }
    
    public Object next() {
        return setSelectedIndex(nextIndex());
    }
    
    public Object previous() {
        return setSelectedIndex(previousIndex());
    }
    
    public IAtomContainer getAtomContainer(int index) {
        if (file == null) return null;
        return file.getAtomContainer(filtered.get(index).intValue());
    }
    
    public int getSelectedIndex() {
        return selectedIndex;
    }
    
    public Object setSelectedIndex(int index) {
    	if (index <= filtered.size()) {
	        selectedIndex = index;
	        return file.setSelectedIndex(filtered.get(index).intValue());
    	} else return null;    
        //return file.getAtomContainer(fpIndex);
    }
    
    public boolean hasNext() {
        if (file == null) return false;
        return selectedIndex<(filtered.size()-1);
    }
    
    public boolean hasPrevious() {
        if (file == null) return false;
        return selectedIndex>0;
    }
    
    public int nextIndex() {
        return getSelectedIndex()+1;
    }
    
    public int previousIndex() {
        return getSelectedIndex()-1;
    }
    
    public Hashtable getProperties(int index) {
        if (file == null) return null;
        return file.getProperties(filtered.get(index).intValue());
    }
    
    public Object getProperty(int index, Object arg0) {
        if (file == null) return null;
        else
        	if (index >= filtered.size()) return null;
        	else
        		return file.getProperty(filtered.get(index).intValue(), arg0);
    }
    public void setProperty(int index, Object key, Object value) {
        if (file == null) return;
    	file.setProperty(filtered.get(index).intValue(), key, value);
    }
    public void set(Object o) {
    	throw new UnsupportedOperationException("Modifying objects not supported!");
    	
    }
    public void remove() {
    	throw new UnsupportedOperationException("Removing objects not supported!");
    	
    }
    public void add(Object o) {
    	throw new UnsupportedOperationException("Adding objects not supported!");
    	
    }
    @Override
    public String toString() {
    	try {
    	return file.toString() + " " + filter.toString();
    	} catch (Exception x) {
    		return x.getMessage();
    	}
    }
    @Override
    public boolean equals(Object obj) {
        if (obj ==null) return false;
        if (obj instanceof FilteredMoleculesFile)
            return obj.toString().equals(toString());
            else return false;

    }
    public void close() {
    	if (file instanceof MoleculesFile) {
	        ((MoleculesFile)file).removePropertyChangeListener(this);
	        ((MoleculesFile)file).firePropertyChange(
	                new DatasetChangeEvent(file,DatasetChangeEvent.op_clear,null,this));
    	}
        file = null;
        filtered.clear();
    }
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(DatasetChangeEvent.op_clear)) {
            if (evt.getSource() == file) {
                close();
            }
        }
        
    }
    public PropertyTranslator getAvailableProperties() {
    	if (file == null) return null;
    	return file.getAvailableProperties();
    }
    public void addAvailableProperties(PropertyTranslator dictionary) {
    	// TODO Auto-generated method stub
    	
    }
    public void clearAvailableProperties() {
        // TODO Auto-generated method stub
        
    }
    public int indexOf(Object property,Object value) throws Exception {
        if ((sortedProperties.size()==0) || (!sortedProperties.getPropertyName().equals(property))) {
            sortedProperties.setPropertyName(property);
            sortedProperties.addList(this);
        }    
        return sortedProperties.getOriginalIndexOf(value);
    }
    public void sortBy(Object property, boolean ascending) throws Exception {
        if ((sortedProperties.size()==0) || (!sortedProperties.getPropertyName().equals(property))) {
            sortedProperties.clear();
            sortedProperties.setPropertyName(property);
            sortedProperties.addList(this);
        }    
        sortedProperties.sort(ascending);
        System.out.println(sortedProperties);
    }
    public Collection<Comparable> getValuesPerField(Object field, int limit) {
        ArrayList<Comparable> values = new ArrayList<Comparable>();
        for (int i=0; i < getAtomContainerCount();i++) {
            Object o = getProperty(i, field);
            if (o==null) continue;
            Comparable c=null;
            if (o instanceof Comparable) c = (Comparable)o;
            else c = o.toString();
            if (!values.contains(c)) values.add(c); 
            if (values.size()==limit) break;
        }
        return values;
    }
}
