/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.openscience.cdk.interfaces.IChemObject;

import ambit.data.IAmbitEditor;
import chemCast.exception.FilterException;
import chemCast.ui.ObjectPropertyFilterEditor;

public class ObjectPropertyFilter extends AbstractFilter implements IPropertyFilter {
	public static String p_tag = "tag";
	public static String p_value="value";
	public ObjectPropertyFilter() throws FilterException {
		super();
	}
	public ObjectPropertyFilter(Object key, Object value) throws FilterException {
		this();
        if ((key ==null) || (key.equals(""))) 
            throw new FilterException("The condition \""+key+"\"=\""+value+"\" can not be created!");
		setProperty(p_tag, key);
		setProperty(p_value, value);
	}
    
	@Override
	public void setProperties(Map properties) {
		
		Iterator i = properties.keySet().iterator();
		while (i.hasNext()) {
			Object key = i.next();
			setProperty(key.toString(), properties.get(key));
		}
	}
	
	@Override
	public void setProperty(String key, Object value) {
		if (p_tag.equals(key) || p_value.equals(key))
			super.setProperty(key, value);
		//else ignore
	}
	public boolean accept(IChemObject object) {
		Object key = parameters.get(p_tag);
		return acceptProperty(null, object.getProperty(key));
	}

	/* (non-Javadoc)
	 * @see chemCast.data.IPropertyFilter#accept(java.lang.Object, java.lang.Object)
	 */
	public boolean acceptProperty(Object property, Object value) {
		try {
            Object values = parameters.get(p_value);
            if (values instanceof Collection)
                return ((Collection)values).contains(value.toString());
            else return values.toString().equals(value.toString());
		} catch (Exception x) {
			x.printStackTrace();
			return false;
		}
		
		//return this.value.equals(value);
	}	

	@Override
	public String toString() {
		return parameters.toString();
	}
	public Object getTag() {
		return getProperty(p_tag);
	}
	public void setTag(Object property) {
		setProperty(p_tag,property);
		
	}
	public Object getValue() {
		return getProperty(p_value);
	}
	public void setValue(Object property) {
		setProperty(p_value,property);
		
	}
	@Override
	public boolean equals(Object arg0) {
		ObjectPropertyFilter pf = (ObjectPropertyFilter) arg0;
		return getTag().equals(pf.getTag())  && getValue().equals(pf.getValue());
	}
	@Override
	public IAmbitEditor getEditor() {
		return new ObjectPropertyFilterEditor("",this);
	}
}

