/* ChemCastObject.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ambit.data.IAmbitEditor;
import ambit.exceptions.AmbitException;
import chemCast.events.PropertyEnabledChangeEvent;

public abstract class ChemCastObject implements Comparable, IChemCastObject, Cloneable {
    protected boolean enabled = true;
    public ChemCastObject(Element element) throws Exception {
        this();
        setXMLElement(element);
    }
    protected PropertyChangeSupport propertyChangeSupport;
    public ChemCastObject() {
        propertyChangeSupport = new PropertyChangeSupport(this);
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#getPropertyChangeSupport()
     */
    public synchronized PropertyChangeSupport getPropertyChangeSupport() {
        return propertyChangeSupport;
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#setPropertyChangeSupport(java.beans.PropertyChangeSupport)
     */
    public synchronized void setPropertyChangeSupport(
            PropertyChangeSupport propertyChangeSupport) {
        this.propertyChangeSupport = propertyChangeSupport;
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#getXMLElement(org.w3c.dom.Document)
     */
    public abstract Element getXMLElement(Document document);
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#setXMLElement(org.w3c.dom.Element)
     */
    public abstract void setXMLElement(Element element) throws Exception;
    
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#firePropertyChange(java.lang.String, java.lang.Object, java.lang.Object)
     */
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    public void firePropertyChange(PropertyChangeEvent event) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.firePropertyChange(event);
        
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     */
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.addPropertyChangeListener(propertyName, listener);
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#addPropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.addPropertyChangeListener(listener);
    }    
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#removePropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.removePropertyChangeListener(listener);
        
    }
 
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastObject#removePropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
     */
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (propertyChangeSupport != null) 
            propertyChangeSupport.removePropertyChangeListener(propertyName,listener);
        
    }
    public void clear() {
        firePropertyChange("clear", null, this);
    }
    public synchronized boolean isEnabled() {
        return enabled;
    }
    public synchronized void setEnabled(boolean enabled) {
    	boolean oldValue = this.enabled;
    	this.enabled = enabled;
        firePropertyChange(
                new PropertyEnabledChangeEvent(this,"enable",oldValue,enabled));
        
    }
    public IAmbitEditor getEditor() {
        return new DefaultChemCastEditor(toString(),this);
    }
}

