/* Endpoint.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class Endpoint extends ChemCastObject {
    public static String p_endpoint = "endpoint";
    protected String name;
    protected SimilarityDefs similarityMethods;
    protected List<Subsets> subsets;
    protected DatasetDef trainingData;
    protected DatasetDef testData;
    
    public Endpoint() throws Exception {
        this("Default",new DatasetDef(),new SimilarityDefs() ,new ArrayList<Subsets>());
    }
    public Endpoint(Element element) throws Exception {
        this();
        setXMLElement(element);
    }
    public Endpoint(String name, DatasetDef source, SimilarityDefs similarityMethods, List<Subsets> subsets) throws Exception {
        super();
        setName(name);
        if (source == null) throw new Exception("Source not defined");
        setTrainingData(source);
        if (similarityMethods == null) throw new Exception("Similarity methods not defined");
        setSimilarityMethods(similarityMethods);
        if (subsets == null) throw new Exception("Subsets not defined");
        setSubsets(subsets);
        setTestData(null);
    }
    @Override
    public Element getXMLElement(Document document) {
        Element e = document.createElement(p_endpoint);
        e.setAttribute(p_name, getName());
        e.appendChild(getTrainingData().getXMLElement(document));
        
        
        Iterator<Subsets> i = subsets.iterator();
        while (i.hasNext()) {
            e.appendChild(i.next().getXMLElement(document));    
        }
        
        e.appendChild(getSimilarityMethods().getXMLElement(document));
        return e;
    }
    @Override
    public String toString() {
        //return "<html>" + getName() + " (<i>" + getSource() + "<i>)" + "</html>";
        return getName();
        /*
        StringBuffer b = new StringBuffer();
        b.append("Name\t");
        b.append(getName());
        b.append('\n');
        b.append("Source\t");
        b.append(getSource());
        b.append('\n');
        b.append("Similarity methods\t");
        b.append(getSimilarityMethods());
        b.append('\n');
        b.append("Subsets\t");
        b.append(getSubsets());
        b.append('\n');
        return b.toString();
        */
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Endpoint) {
            Endpoint e = (Endpoint) obj;
            return getName().equals(e.getName()) &&
            getTrainingData().equals(e.getTrainingData()) &&
            getSubsets().equals(e.getSubsets()) &&
            getSimilarityMethods().equals(e.getSimilarityMethods());
        } else return false;
    }
    @Override
    public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_endpoint)) {
            setName(element.getAttribute(IChemCastObject.p_name));
            //element.appendChild(getSource().setXMLElement(document));
            NodeList nodes = element.getElementsByTagName(DatasetDef.p_source);
            getTrainingData().clear();
            if (nodes != null) 
                for (int i=0; i < nodes.getLength(); i++) {
                    setTrainingData(new DatasetDef((Element)nodes.item(i)));
                }     
                

            
            nodes = element.getElementsByTagName(Subsets.p_groups);
            getSubsets().clear();
            if (nodes != null) 
                for (int i=0; i < nodes.getLength(); i++) {
                    addSubsets(new Subsets((Element)nodes.item(i)));
                }     
                
            //System.out.println(getName());
            nodes = element.getElementsByTagName(SimilarityDefs.p_similarityMethods);
            if (nodes != null)
                for (int i=0; i < nodes.getLength(); i++) {
                    setSimilarityMethods(new SimilarityDefs((Element)nodes.item(i)));
                    break;
                }     
            else getSubsets().clear();
        } else throw new Exception("Not an element of type "+p_endpoint);

    }

    public int compareTo(Object o) {
        // TODO Auto-generated method stub
        return 0;
    }
    public synchronized String getName() {
        return name;
    }
    public synchronized void setName(String name) {
        firePropertyChange(p_name, this.name,name);
        this.name = name;
    }
    public synchronized SimilarityDefs getSimilarityMethods() {
        return similarityMethods;
    }
    public synchronized void setSimilarityMethods(SimilarityDefs similarityMethods) {
        firePropertyChange(SimilarityDefs.p_similarityMethods, this.similarityMethods,similarityMethods);
        this.similarityMethods = similarityMethods;
    }
    public synchronized DatasetDef getTrainingData() {
        return trainingData;
    }
    public synchronized void setTrainingData(DatasetDef source) {
        firePropertyChange(DatasetDef.p_source, this.trainingData,source);
        this.trainingData = source;
        
    }
    public synchronized List<Subsets> getSubsets() {
        return subsets;
    }
    public synchronized void setSubsets(List<Subsets> subsets) {
        firePropertyChange(Subsets.p_groups, this.subsets, subsets);
        this.subsets = subsets;
    }
    public synchronized void addSubsets(Subsets subsets) {
        this.subsets.add(subsets);
        firePropertyChange(Subsets.p_groups, this.subsets, subsets);
    }
    public synchronized void removeSubsets(Subsets subsets) {
        this.subsets.remove(subsets);
        firePropertyChange(Subsets.p_groups, this.subsets, subsets);
    }
	public DatasetDef getTestData() {
		return testData;
	}
	public void setTestData(DatasetDef testData) {
        firePropertyChange(DatasetDef.p_query, this.testData,testData);
        this.testData = testData;		
	}    
}
