/* MCSSSimilarity.java
 * Author: Nina Jeliazkova
 * Date: Nov 20, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.List;

import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.isomorphism.UniversalIsomorphismTester;
import org.openscience.cdk.qsar.model.QSARModelException;

import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.similarity.DefaultSimilarityProcessor;
import chemCast.weka.ISubsetsModel;

public class MCSSSimilarity extends DefaultSimilarityProcessor implements
        ISubsetsModel {
    protected Subsets datasets = null;
    /**
     * 
     */
    private static final long serialVersionUID = 1483299277730546579L;

    public MCSSSimilarity() {
        super();
    }

    public Subsets getDatasets() {
        return datasets;
    }

    public float predict(IAtomContainersList object1, int index1,
            IAtomContainersList object2, int index2) throws AmbitException {
        try {
        	
            IAtomContainer a1 = object1.getAtomContainer(index1);
            IAtomContainer a2 = object2.getAtomContainer(index2);
            List commonSubstructures =  UniversalIsomorphismTester.getOverlaps(a1,a2);
            
            int max = -1;
            IAtomContainer mcss = null;
            for (int i = 0; i < commonSubstructures.size(); i++){
               IAtomContainer a = (IAtomContainer)commonSubstructures.get(i);
               if ((i==0) || (a.getAtomCount() > max)) {
                  max = a.getAtomCount();
                  mcss = a;
               }
            }
            float mcssAB = mcss.getAtomCount() + mcss.getElectronContainerCount();
            float c1 = a1.getAtomCount() + a1.getElectronContainerCount();
            float c2 = a2.getAtomCount() + a2.getElectronContainerCount();
            if ((c1>0)&&(c2>0)) {
                return (mcssAB*mcssAB)/(c1*c2);
            }
        } catch (CDKException x) {
            x.printStackTrace();
        }
        return 0;

    }

    public void setDatasets(Subsets datasets) {
        this.datasets = datasets;

    }

    public void build() throws QSARModelException {
        // TODO Auto-generated method stub

    }

    public void predict() throws QSARModelException {
        // TODO Auto-generated method stub

    }

    public Object getParameter(Object parameter) {
        // TODO Auto-generated method stub
        return null;
    }

    public void setParameter(Object parameter, Object value) {
        // TODO Auto-generated method stub

    }
    @Override
    public String toString() {
        return "Maximum Common Substructure";
    }
}
