/* SimilarityDef.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.Enumeration;
import java.util.Hashtable;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ambit.data.IAmbitEditor;
import ambit.similarity.ISimilarityProcessor;

public class SimilarityDef extends ChemCastObject {
    public static String p_similarity = "similarity";
    public static String p_classname = "classname";
    public static String p_type = "type";
    public static String p_hasgroups = "hasgroups";
    protected String name;
    protected String type;
    protected String groups;
    protected String className;
    protected ChemCastDescriptorsList descriptors = null;
    protected Hashtable<String,String> parameters=null;
    
    public SimilarityDef() {
        this("Fingerprints/Tanimoto","Structure","","ambit.similarity.FingerprintSimilarity");
    }
    public SimilarityDef(String name, String type, String groups, String classname) {
        super();
        setName(name);
        setType(type);
        setGroups(groups);
        setClassName(classname);
        parameters = new Hashtable<String,String>();
    }
    public SimilarityDef(Element element) throws Exception {
        this();
        setXMLElement(element);
    }
    @Override
    public Element getXMLElement(Document document) {
        Element e = document.createElement(p_similarity);
        e.setAttribute(p_name, getName());
        e.setAttribute(p_type, getType());
        e.setAttribute(p_hasgroups, getGroups());
        e.setAttribute(p_classname, getClassName());
        if (descriptors != null)
            e.appendChild(descriptors.getXMLElement(document));
        
        if (parameters.size()>0) {
            Element eparameters = document.createElement(p_parameters);
            Enumeration i = parameters.keys();
            while (i.hasMoreElements()) {
                Element parameter = document.createElement(p_parameter);
                Object key = i.nextElement();
                parameter.setAttribute(p_name,key.toString());
                parameter.setAttribute("value", parameters.get(key).toString());
                eparameters.appendChild(parameter);
            }        
            e.appendChild(eparameters);
        }
        return e;
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof SimilarityDef) {
            SimilarityDef s = (SimilarityDef) obj;
            boolean ok = s.getName().equals(getName()) &&
                    s.getType().equals(getType()) &&
                    s.getClassName().equals(getClassName());
            if (ok) {
            	Hashtable params = s.getParameters();
            	if (params.size() != parameters.size()) return false;
            	return params.equals(parameters);
            } 
        } 
        return false;
    }
    /*
     * Here are some useful guidelines for implementing the hashCode method correctly.
	
	Store an arbitrary non-zero constant integer value (say 7) in an int variable, called hash. 
	Involve significant variables of your object in the calculation of the hash code, all the variables that are part of equals comparison should be considered for this. Compute an individual hash code int var_code for each variable var as follows - 
	If the variable(var) is byte, char, short or int, then var_code = (int)var; 
	If the variable(var) is long, then var_code = (int)(var ^ (var >>> 32)); 
	If the variable(var) is float, then var_code = Float.floatToIntBits(var); 
	If the variable(var) is double, then -
	long bits = Double.doubleToLongBits(var);
	var_code = (int)(bits ^ (bits >>> 32)); 
	If the variable(var) is boolean, then var_code = var ? 1 : 0; 
	If the variable(var) is an object reference, then check if it is null, if yes then var_code = 0; otherwise invoke the hashCode method recursively on this object reference to get the hash code. This can be simplified and given as -
	var_code = (null == var ? 0 : var.hashCode()); 
	Combine this individual variable hash code var_code in the original hash code hash as follows - 
	hash = 31 * hash + var_code; 
	Follow these steps for all the significant variables and in the end return the resulting integer hash. 
	Lastly, review your hashCode method and check if it is returning equal hash codes for equal objects. Also, verify that the hash codes returned for the object are consistently the same for multiple invocations during the same execution. 
	(non-Javadoc)
	     * @see java.lang.Object#hashCode()
	     */
    @Override
    public int hashCode() {
    	int hash = 7;
    	int var_code = (null == name ? 0 : name.hashCode());
    	hash = 31 * hash + var_code; 
    	var_code = (null == type ? 0 : type.hashCode());
    	hash = 31 * hash + var_code; 
    	var_code = (null == groups ? 0 : groups.hashCode());
    	hash = 31 * hash + var_code; 
    	var_code = (null == className ? 0 : className.hashCode());
    	hash = 31 * hash + var_code;

    	var_code = (null == descriptors ? 0 : descriptors.hashCode());
    	hash = 31 * hash + var_code; 
	
    	return hash;
    }
    @Override
    public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_similarity)) {
            setName(element.getAttribute(IChemCastObject.p_name));
            setType(element.getAttribute(p_type));
            setClassName(element.getAttribute(p_classname));
            setGroups(element.getAttribute(p_hasgroups));
            NodeList nodes = element.getElementsByTagName(ChemCastDescriptorsList.p_descriptors);
            for (int i=0; i < nodes.getLength(); i++) {
                ChemCastDescriptorsList d = new ChemCastDescriptorsList((Element)nodes.item(i));
                setDescriptors(d);
            }
            //System.out.println(getName());
            
            NodeList child = element.getElementsByTagName(p_parameters);
            for (int i=0; i<child.getLength();i++) {
                Node node = child.item(i);
                if (node instanceof Element) {
                    NodeList param = ((Element)node).getElementsByTagName(p_parameter);
                    for (int j=0; j<param.getLength();j++) {
                        String name = ((Element)param.item(j)).getAttribute(p_name);
                        String value = ((Element)param.item(j)).getAttribute("value");
                        parameters.put(name,value);
             //           System.out.println(name + "," + value);
                    }
                }
            }
            
            //System.out.println(parameters);
            //System.out.println();
        } else throw new Exception("Not an element of type "+p_similarity);

    }

    public int compareTo(Object o) {
        // TODO Auto-generated method stub
        return 0;
    }
    public synchronized String getClassName() {
        return className;
    }
    public synchronized void setClassName(String className) {
        firePropertyChange(p_classname, this.className,className);
        this.className = className;
    }
    public synchronized void setGroups(String groups) {
        firePropertyChange(p_hasgroups, this.groups,groups);
        this.groups = groups;
    }
    public synchronized String getName() {
        return name;
    }
    public synchronized void setName(String name) {
        firePropertyChange(p_name, this.name,name);
        this.name = name;
    }
    public synchronized String getType() {
        return type;
    }
    public synchronized void setType(String type) {
        firePropertyChange(p_type, this.type,type);
        this.type = type;
    }
    public synchronized String getGroups() {
        return groups;
        
    }
    @Override
    public String toString() {
        return getName();
    }
    public ISimilarityProcessor createSimilarityProcessor() throws Exception {
    	Object processor = Class.forName(getClassName()).newInstance();
    	if (processor instanceof ISimilarityProcessor) {
            ((ISimilarityProcessor) processor).setParameter(ChemCastDescriptorsList.p_descriptors, getDescriptors());
            //parameters
            Enumeration i = parameters.keys();
            while (i.hasMoreElements()) {
                Object key = i.nextElement();
                ((ISimilarityProcessor) processor).setParameter(key.toString(), parameters.get(key).toString());
            }
            return (ISimilarityProcessor) processor;
        }
    	else throw new Exception("Not an instance of Similarity Processor, but " + (processor.getClass().getName()));

    }
    public synchronized ChemCastDescriptorsList getDescriptors() {
        return descriptors;
    }
    public synchronized void setDescriptors(ChemCastDescriptorsList descriptors) {
        this.descriptors = descriptors;
    }
    public synchronized Hashtable<String, String> getParameters() {
        return parameters;
    }
    public synchronized void setParameters(Hashtable<String, String> parameters) {
        this.parameters = parameters;
    }
    @Override
    public IAmbitEditor getEditor() {
        return new SimilarityDefEditor(this);
    }
}


