/* SimilaritySubsets.java
 * Author: Nina Jeliazkova
 * Date: Oct 22, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.openscience.cdk.io.listener.IReaderListener;
import org.w3c.dom.Element;

import chemCast.exception.SubsetException;

public class SimilaritySubsets extends Subsets {
    
    //protected Hashtable<SimilarityModel,ISimilarityProcessor> processors;
	//protected ArrayList<SimilarityModel> processors;
    protected SimilarityModels processors;
    //protected ISimilarityProcessor selectedProcessor;
	protected SimilarityModel selectedProcessor;
    protected PropertyChangeListener resultListener = null;
    protected IReaderListener readerListener = null;

    public SimilaritySubsets() {
        super();
        init();
    }

    public SimilaritySubsets(String type, String name) {
        super(type, name);
        init();
    }

    public SimilaritySubsets(Element element) throws Exception {
        super(element);
        init();
    }
    protected void init() {
        
        processors = new SimilarityModels();
        selectedProcessor = null;
 
    }
    @Override
    public void clear() {
        fields.clear();
        firePropertyChange(IChemCastObject.p_properties,null,fields);
        clearSelectedProcessor();

        Iterator i = processors.getList().iterator();
        while (i.hasNext()) 
            ((SimilarityModel)i.next()).clear();
        processors.clear();
        super.clear();
    }
    @Override
    /**
     * Calls the inherited method and updates fields. 
     */
    public void createSubsets(DatasetDef dataset, Subsets subsets) throws SubsetException {
        super.createSubsets(dataset, subsets);
        fields.add(dataset.getDatasetResultField());
        dataset.getProperties().addProperty("endpoint", dataset.getDatasetResultField(),dataset.getDatasetResultField());
        addFields(dataset.getProperties().getIdentifiers().values());
        firePropertyChange(IChemCastObject.p_properties,null,fields);
        for (int i=0; i < subsets.size();i++) {
            Object tag = subsets.get(i).getProperty("tag");
            if (tag != null)
                dataset.getProperties().addProperty("subsets", tag, tag);

        }    
    }
    /*
     *        if (enew != null) {
            List list = enew.getList();
            for (int i=0; i < list.size();i++)
                propertyChangeSupport.firePropertyChange(IChemCastObject.p_property, null, list.get(i).toString());
        } 
     */
    protected void clearSelectedProcessor() {
            if ((selectedProcessor != null) && (selectedProcessor.getProcessor()!=null))  {
            	selectedProcessor.getProcessor().firePropertyChange("Result", null,null);
                selectedProcessor.getProcessor().removePropertyChangeListener("Result", 
                        resultListener);
            }    

            selectedProcessor = null;
    }
    public void setSimilarity(SimilarityModel similarity) throws Exception {
        propertyChangeSupport.firePropertyChange(SimilarityDef.p_similarity,null,similarity);
        clearSelectedProcessor();

        try {
        	int pindex = processors.indexOf(similarity);
        	if (pindex >=0) {
        		selectedProcessor = processors.get(pindex);
                selectedProcessor.getProcessor().addPropertyChangeListener(resultListener);
        	} else {
        		selectedProcessor = similarity;
        		selectedProcessor.createSimilarityProcessor();
        		selectedProcessor.getProcessor().setSimilarityProperty(similarity.getName());
        		processors.add(similarity);
                selectedProcessor.getProcessor().addPropertyChangeListener(resultListener);
        	}
        	
     
        } catch (Exception x) {
            x.printStackTrace();
            selectedProcessor = null;
            throw new Exception("Can't use similarity method '"+ similarity + "'!\n Can't create "+similarity.getClassName(),x);
        }
   
    }

    public synchronized SimilarityModel getSelectedProcessor() {
        return selectedProcessor;
    }
    public synchronized PropertyChangeListener getResultListener() {
        return resultListener;
    }

    public synchronized void setResultListener(PropertyChangeListener resultListener) {
        this.resultListener = resultListener;
    }

    public synchronized IReaderListener getReaderListener() {
        return readerListener;
    }

    public synchronized void setReaderListener(IReaderListener readerListener) {
        this.readerListener = readerListener;
    }

    public synchronized SimilarityModels getProcessors() {
        return processors;
    }

    public synchronized void setProcessors(SimilarityModels processors) {
        this.processors = processors;
    }



}
