/* Subsets.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.MoleculesIterator;
import chemCast.data.IObjectFilter;
import chemCast.data.SelectedItem;
import chemCast.events.SelectItemEvent;
import chemCast.exception.SubsetException;

public class Subsets extends ChemCastObject implements IChemCastList, Comparable {
    protected ArrayList<String> fields;
    public static String p_groups="groups";
    public static String p_type="type";
    protected String type ="set";
    protected List<Subset> list;
    protected String name = "N/A";
    protected String filterClass = "chemCast.data.PropertyFilter";

    
    public Subsets() {
        super();
        list = new ArrayList<Subset>();
        fields = new ArrayList<String>();
    }
    public Subsets(String type,String name) {
        this();
        setName(name);
        setType(type);
    }
    public Subsets(Element element) throws Exception {
        this();
        setXMLElement(element);
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastList#size()
     */
    public int size() {
        return list.size();
    }
    @Override
    public Element getXMLElement(Document document) {
        Element e = ChemCastObjectFactory.getXMLElement(list, document, p_groups);
        e.setAttribute(p_name, getName());
        if (filterClass!= null)
        e.setAttribute(p_class, filterClass);
        return e;
    }

    @Override
    public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_groups)) {
            NodeList nodes = element.getElementsByTagName(Subset.p_group);
            setName(element.getAttribute(IChemCastObject.p_name));
            setFilterClass(element.getAttribute(IChemCastObject.p_class));
            for (int i=0; i < nodes.getLength(); i++) {
                Subset s = new Subset((Element)nodes.item(i));
                if ((s.getFilterClass() == null) || s.getFilterClass().equals(""))
                		s.setFilterClass(filterClass);
                list.add(s);
            }
        }

    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastList#add(chemCast.similarity.data.Subset)
     */
    public void add(Subset subset) {
        list.add(subset);
        propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
        /*
        propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                this,
                type,
                null,
                new SelectedItem(this,subset,0)
                ));
        */
    }
    public Subset get(int index) {
    	if ((index >=0) && (index< size()))
    		return list.get(index);
    	else return null;
     
    }
    public void add(Subsets subset) {
        for (int i=0; i < subset.getList().size();i++)
            list.add(subset.get(i));
        propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);

    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastList#remove(chemCast.similarity.data.Subset)
     */
    public void remove(Subset subset) {
        list.remove(subset);
        propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
        /*
        propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                this,
                type,
                new SelectedItem(this,subset,0),
                null
                ));
                */        
    }
    /* (non-Javadoc)
     * @see chemCast.similarity.data.IChemCastList#clear()
     */
    public void clear() {
        propertyChangeSupport.firePropertyChange(Subsets.p_groups, this,null);
        for (int i=size()-1; i >= 0; i--) if (get(i) != null) get(i).clear();
        list.clear();
        super.clear();
        propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                this,
                type,
                null,
                null
                ));  
    }
    public int compareTo(Object o) {
        return toString().compareTo(o.toString());
    }
    public List <? extends IChemCastObject> getList() {
        return list;
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Subsets) {
            return list.equals(((Subsets) obj).getList());
        } return super.equals(obj);
    }
    @Override
    public String toString() {
        return getName();//return ChemCastObjectFactory.toString(options);
    }
    public synchronized String getName() {
        return name;
    }
    public synchronized void setName(String name) {
        firePropertyChange(p_name, this.name,name);
        this.name = name;
    }
    /**
     * TODO fire properties
     * @param subsets
     * @throws SubsetException
     */
    public void updateSubsets(Subsets subsets) throws SubsetException {
        Subset parentDataset = null;
        for (int j=0; j < size(); j++) 
            if (get(j) instanceof DatasetDef) {
                parentDataset = get(j);
                list.set(j,null);
                break;
            }
        if (parentDataset == null) throw new SubsetException("No parent dataset! Can't create subsets.");
        createSubsets((DatasetDef)parentDataset, subsets);
    }
    public void createSubsets(IAtomContainersList dataset, Subsets subsets) throws SubsetException {
        DatasetDef subset = new DatasetDef(dataset.toString(),"","");
        subset.setData(dataset);
        subset.setColor(Color.blue);
        subset.setEnabled(false);
        createSubsets(subset, subsets);
        
    }
    public void createNew() throws SubsetException {
        clear();
        try {
            DatasetDef subset = new DatasetDef();
            subset.setData(new MoleculesIterator());
            add(subset);
            propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
            propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                    this,
                    name,
                    null,
                    new SelectedItem(this,subset,0)
                    ));
        }  catch (SubsetException x) {
            x.printStackTrace();
        }
    }
    /**
     * Same as addnewsubset, but does not add to the list.
     * @param subset
     * @throws SubsetException
     */
    public Subset createNewSubset(Subset subset, IObjectFilter filter) throws SubsetException {
        try {
            DatasetDef dataset = getParentDataset();
            if ((dataset==null) || (dataset.getData() == null)) 
                    throw new SubsetException("No dataset available!");
            
            subset.setParentDataset(dataset.getData(),filter);
            subset.setEnabled(false);
            //list.add(s);
            //propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
            //System.out.println("sent " + subset.toString());
            propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                    this,
                    name,
                    null,
                    new SelectedItem(this,subset,0)
                    ));
            return subset;
        } catch (Exception x) {
            throw new SubsetException("Error when creating new subset",x);
        }
    }    
    public void addNewSubset(Subset subset) throws SubsetException {
        try {
            DatasetDef dataset = getParentDataset();
            if ((dataset==null) || (dataset.getData() == null)) 
                    throw new SubsetException("No dataset available!");
            
            Subset s = (Subset)subset.clone();
            s.setParentDataset(dataset.getData(),null);
            s.setEnabled(false);
            list.add(s);
            propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
            propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                    this,
                    name,
                    null,
                    new SelectedItem(this,dataset,0)
                    ));
        } catch (Exception x) {
            throw new SubsetException("Error when creating new subset",x);
        }
    }
    public void createSubsets(DatasetDef dataset, Subsets subsets) throws SubsetException {    
        //delete previous content
        clear();
        //adding parent dataset
        list.add(dataset);
        dataset.setEnabled(true);
            //add subsets
            for (int j=0; j < subsets.size(); j++) {
                try {
                    Subset s = (Subset)subsets.get(j).clone();
                    s.setParentDataset(dataset.getData(),null);
                    s.setEnabled(false);
                    list.add(s);
                } catch (Exception x) {
                    x.printStackTrace();
                }
            }
            propertyChangeSupport.firePropertyChange(Subsets.p_groups, null, this);
            propertyChangeSupport.firePropertyChange(new SelectItemEvent(
                    this,
                    name,
                    null,
                    new SelectedItem(this,dataset,0)
                    ));


    }
    public synchronized String getType() {
        return type;
    }
    public synchronized void setType(String type) {
        firePropertyChange(p_type, this.type,type);
        this.type = type;
    }
	public String getFilterClass() {
		return filterClass;
	}
	public void setFilterClass(String filterClass) {
		firePropertyChange(p_class, this.filterClass,filterClass);
		this.filterClass = filterClass;
	}
    public DatasetDef getParentDataset() {
        DatasetDef dataset = null;
        for (int i=0;i<size();i++)
            if (get(i) instanceof DatasetDef) {
                dataset = (DatasetDef) get(i);
                break;
            }    
        if (dataset == null) return null;
        try {
            if (dataset.getData() == null) return null;
        } catch (Exception x) {
            return null;
        }
        return dataset;
    }
    public synchronized ArrayList<String> getFields() {
        return fields;
    }
    public void addFields(Collection moreFields) {
        fields.addAll(moreFields);
        firePropertyChange(IChemCastObject.p_properties,null,fields);
    }

    public synchronized void setFields(ArrayList<String> fields) {
        this.fields = fields;
        firePropertyChange(IChemCastObject.p_properties,null,fields);
    }
    public void selectAll(boolean selected) {

    	
    }
}
