/* PropertyImporterTest.java
 * Author: Nina Jeliazkova
 * Date: Nov 9, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.data;

import java.io.File;
import java.io.FileInputStream;

import junit.framework.TestCase;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.iterator.IIteratingChemObjectReader;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import ambit.data.molecule.PropertyTranslator;
import ambit.io.FileInputState;
import ambit.io.MolPropertiesIOSetting;
import chemCast.data.BatchDescriptorImporter;
import chemCast.data.MoleculesFile;

public class PropertyImporterTest extends TestCase {
    PropertyTranslator fileProperties;
    BatchDescriptorImporter batch;
    public PropertyImporterTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }
    public void test() {
        try {
            fileProperties = new PropertyTranslator();
            PropertyTranslator properties = new PropertyTranslator();
            properties.addProperty(PropertyTranslator.type_identifiers,"CASRN", CDKConstants.CASRN);
            properties.addProperty(PropertyTranslator.type_descriptors,"1", "1");
            properties.addProperty(PropertyTranslator.type_descriptors,"2", "2");
            File file = new File("data/descriptors_import_test.csv");
            IIteratingChemObjectReader reader = FileInputState.getReader(
                    new FileInputStream(file), file.getName());
            IReaderListener listener = new IReaderListener() {
                public void processIOSettingQuestion(IOSetting setting) {
                    if (setting instanceof MolPropertiesIOSetting) 
                        fileProperties.addProperty(PropertyTranslator.type_identifiers,
                                "TestSubstance_CASRN", CDKConstants.CASRN);
                 }
                public void frameRead(ReaderEvent arg0) {
                }
            };
            MoleculesFile mf = new MoleculesFile(new File("data/EPAFHM_v3b_617_10Apr2006.sdf"),DefaultChemObjectBuilder.getInstance(),listener);
            mf.addAvailableProperties(fileProperties);
            assertEquals("",mf.getProperty(0, "1"));
            batch = new BatchDescriptorImporter(reader,mf,properties,CDKConstants.CASRN);
            reader.addChemObjectIOListener(new IReaderListener() {
                public void processIOSettingQuestion(IOSetting setting) {
                if (setting instanceof MolPropertiesIOSetting) {
                    batch.updateProperties(((MolPropertiesIOSetting)setting).getProperties(),"");
                }
                }
                public void frameRead(ReaderEvent arg0) {
                    
                }
                
            });
            batch.start();
            reader.close();
            
            assertEquals("3",mf.getProperty(0, "1"));
            mf.close();
            
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }

    }
}
