/* DiceSimilarityTest.java
 * Author: Nina Jeliazkova
 * Date: Nov 4, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.weka;

import junit.framework.TestCase;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.KDTree;
import chemCast.weka.DiceSimilarity;

public class DiceSimilarityTest extends TestCase {

    public DiceSimilarityTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testPostProcessDistances() {
        DiceSimilarity ts = new DiceSimilarity();
        double[] olddistances = new double[] {0,1,0.2};
        double[] distances = new double[] {0,1,0.2};
        ts.postProcessDistances(distances);
        for (int i=0; i < distances.length;i++)
            assertEquals(olddistances[i],distances[i]);
    }

    public void testDistanceInstanceInstanceDoubleBoolean() {
        FastVector attributes = new FastVector();
        attributes.addElement(new Attribute("FP1"));
        attributes.addElement(new Attribute("FP2"));
        Instances data = new Instances("Test",attributes,1);
        
        Instance a = new Instance(1,new double[] {1,1});
        Instance b = new Instance(1,new double[] {1,0});
        Instance c = new Instance(1,new double[] {0,1});
        Instance d = new Instance(1,new double[] {0,0});
        Instance e = new Instance(1,new double[] {0.2,1.7});  
        data.add(a);a.setDataset(data);
        data.add(b);b.setDataset(data);
        
        DiceSimilarity ts = new DiceSimilarity();
        ts.setInstances(data);
        try {
            //2*Zab/(Zaa+Zbb)
            double t = 2*(1*0.2 + 0*1.7) / ((1*1+0)+(0.2*0.2+1.7*1.7));
            
            assertEquals(1-(2.0*1.0/(2.0+1.0)),ts.distance(a, b));
            assertEquals(1-(2.0*1.0/(2.0+1.0)),ts.distance(a,c));
            assertEquals(1-t,ts.distance(b,e));
            assertEquals(1.0,ts.distance(a,d));
            assertEquals(1.0,ts.distance(b,c));
            assertEquals(1.0,ts.distance(b,d));
            
            assertEquals(0.0,ts.distance(d,d));
            assertEquals(0.0,ts.distance(a,a));
            assertEquals(0.0,ts.distance(b,b));
            assertEquals(0.0,ts.distance(c,c));
            assertEquals(0.0,ts.distance(e,e));
  
            
            
            data.add(c); c.setDataset(data);
            data.add(d);d.setDataset(data);
            data.add(e);e.setDataset(data);
            KDTree tree = new KDTree();
            tree.setDistanceFunction(ts);
            tree.setInstances(data);
            Instances nn = tree.kNearestNeighbours(a, data.numInstances());
            double[] dist = {ts.distance(a,a),
                    ts.distance(a,e),ts.distance(a,c),ts.distance(a,b),ts.distance(a,d)};
            for (int i=0;i < nn.numInstances(); i++) {
                assertEquals(dist[i],tree.getDistanceFunction().distance(a, nn.instance(i)));
/*                
                System.out.print(nn.instance(i));
                System.out.print(" d=");
                System.out.println(tree.getDistanceFunction().distance(a, nn.instance(i)));
  */              
                
            }    
            
        } catch (Exception x) {
            fail(x.getMessage());
        }

    }

}
