/*
Copyright Ideaconsult Ltd. (C) 2005-2007  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.InputStream;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpringLayout;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import ambit.io.MyIOUtilities;
import ambit.ui.SpringUtilities;


/**
 * Config file editor
 * @author nina
 *
 */
public class ConfigTree extends JPanel implements TreeSelectionListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 679949827442747266L;
    private AttributeEditor attributeEditor;
    private JTree tree;
    private URL helpURL;
    private static boolean DEBUG = false;
    //"Angled" (the default), "Horizontal", and "None".
    private static boolean playWithLineStyle = false;
    private static String lineStyle = "Horizontal";
    protected Document doc = null;
    protected String configFile;
    protected DefaultMutableTreeNode currentNode = null;
    protected DefaultMutableTreeNode copiedNode = null;
    protected DefaultMutableTreeNode top;
    
	public ConfigTree() {
		super(new BorderLayout());
        top = new DefaultMutableTreeNode("ToxMatch configuration");
        doc = createNodes(top);

        //Create a tree that allows one selection at a time.
        tree = new JTree(top);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for when the selection changes.
        tree.addTreeSelectionListener(this);

        if (playWithLineStyle) {
            System.out.println("line style = " + lineStyle);
            tree.putClientProperty("JTree.lineStyle", lineStyle);
        }

        //Create the scroll pane and add the tree to it. 
        JScrollPane treeView = new JScrollPane(tree);

        //Create the HTML viewing pane.
        attributeEditor = new AttributeEditor();
        
        JScrollPane htmlView = new JScrollPane(attributeEditor);

        //Add the scroll panes to a split pane.
        final JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setTopComponent(treeView);
        splitPane.setBottomComponent(htmlView);

        Dimension minimumSize = new Dimension(100, 50);
        htmlView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(300); 
        splitPane.setPreferredSize(new Dimension(500, 500));


        
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("File");
        menuBar.add(menu);
        menu.add(new AbstractAction("Save") {
        	public void actionPerformed(ActionEvent e) {
        		save(new File(configFile));
        		
        	}
        });
        menu.add(new AbstractAction("Save As") {
        	public void actionPerformed(ActionEvent e) {
        	
        		File file = MyIOUtilities.selectFile(splitPane,
        				"Save configuration file",
        				"", 
        				new String[] {".xml"}, 
        				new String[] {"ToxMatch configuration file (.xml)"},
        				false);
        				
        		save(file);
        		
        	}
        });     
        menu.addSeparator();
        menu.add(new AbstractAction("Restore default") {
        	public void actionPerformed(ActionEvent e) {
        		try {
        			restore();
        		} catch (Exception x) {
        			JOptionPane.showMessageDialog(splitPane, x.getMessage());
        		}
        	}
        });            
        add(menuBar);
        menu = new JMenu("Edit");
        menuBar.add(menu);        
        menu.add(new AbstractAction("Copy") {
        	public void actionPerformed(ActionEvent e) {
        		copiedNode = currentNode;
        	}
        });        
        menu.add(new AbstractAction("Paste") {
        	public void actionPerformed(ActionEvent e) {
        		try {
        			paste();
        		} catch (Exception x) {
        			JOptionPane.showMessageDialog(splitPane, x.getMessage());
        		}
        	}
        });   
        menu.add(new AbstractAction("Delete") {
        	public void actionPerformed(ActionEvent e) {
        		try {
        			delete();
        		} catch (Exception x) {
        			JOptionPane.showMessageDialog(splitPane, x.getMessage());
        		}
        	}
        });            
        add(menuBar,BorderLayout.NORTH);
        //Add the split pane to this panel.
        add(splitPane,BorderLayout.CENTER);

	}
	protected void restore() {
		doc = createDefaultNodes(top);
		((DefaultTreeModel) tree.getModel()).nodeStructureChanged(top);
		
	}
	private Document createNodes(DefaultMutableTreeNode top) {
		try {
	        File file = new File("ChemCAST_default.xml");
	        configFile = file.getAbsolutePath();
	        return createNodes(top, file);
		} catch (FileNotFoundException x) {
			return createDefaultNodes(top);
		}
	}
	private Document createDefaultNodes(DefaultMutableTreeNode top) {
		 //URL url = this.getClass().getClassLoader().getResource("chemCast/ui/images/config.xml");
		 //System.out.println(url);
         InputStream in = ConfigTree.class.getClassLoader().getResourceAsStream("chemCast/ui/images/config.xml");
		 try {
			 if (in != null) {
				 top.removeAllChildren();
				 return createNodes(top, new InputSource(in));
			 } else 
				 System.out.println("Cant't find config file");
		 } catch (Exception x) {
			 x.printStackTrace();
		 } finally {
			 try { in.close();} catch (Exception x) {}
		 }
		 return doc;
        
	}	
	 private Document createNodes(DefaultMutableTreeNode top,File file) throws FileNotFoundException {
		 FileReader reader = null;
		 try {
			 reader = new FileReader(file);
			 return createNodes(top,new InputSource(reader));
		 } catch (FileNotFoundException x) {
			 throw x;
		 } finally {
			 try { reader.close();} catch (Exception x) {}
		 }
	 }
    private Document createNodes(DefaultMutableTreeNode top,InputSource src) {

        Document doc = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(false);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.parse(src);
            
            buildTree(doc.getDocumentElement(), top);
            
        } catch (Exception x) {
            x.printStackTrace();
        } finally {
        	
        }
        return doc;
    }
    protected boolean isPasteAllowed(Node copiedNode,Node parentNode) {
    	return
    	 (copiedNode.getNodeName().equals("endpoint") && parentNode.getNodeName().equals("endpoints")) ||
    	 (copiedNode.getNodeName().equals("descriptor") && parentNode.getNodeName().equals("descriptors")) ||
    	 (copiedNode.getNodeName().equals("group") && parentNode.getNodeName().equals("groups")) ||
    	 (copiedNode.getNodeName().equals("groups") && parentNode.getNodeName().equals("endpoint")) ||
    	 (copiedNode.getNodeName().equals("representation") && parentNode.getNodeName().equals("similarity")) ||
    	 (copiedNode.getNodeName().equals("measure") && parentNode.getNodeName().equals("similarity")) ||
    	 (copiedNode.getNodeName().equals("trainingset") && parentNode.getNodeName().equals("endpoint")) ||
    	 (copiedNode.getNodeName().equals("similarity") && parentNode.getNodeName().equals("similarityMethods")) ||
    	 (copiedNode.getNodeName().equals("identifier") && parentNode.getNodeName().equals("identifiers")) ||
    	 (copiedNode.getNodeName().equals("parameter") && parentNode.getNodeName().equals("parameters")) ;
    }
    
    protected void delete() throws Exception {
    	if (currentNode== null) return;
    	
    	DefaultMutableTreeNode parent = (DefaultMutableTreeNode) currentNode.getParent();

    	if ((parent != null) && (parent.getUserObject() instanceof Element)){
    		((Element) parent.getUserObject()).removeChild((Element)currentNode.getUserObject());
    		parent.remove(currentNode);
    		currentNode = null;
    		copiedNode = null;
    		((DefaultTreeModel) tree.getModel()).nodeStructureChanged(parent);

    	}
    }
    protected void paste() throws Exception {
    	if (copiedNode==null) return;
    	if (currentNode == null) return;
    	if (currentNode.getUserObject() instanceof Element) {
    		
    		if (!isPasteAllowed((Node)copiedNode.getUserObject(),(Node) currentNode.getUserObject())) 
    			throw new Exception("Not allowed");
    		Element newElement = copyElement((Element)copiedNode.getUserObject());
	    	((Element)currentNode.getUserObject()).appendChild(newElement);
	    	
    		DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(newElement) {
    			@Override
    			public String toString() {
    				Element element = (Element)getUserObject();
    				String name = element.getAttribute("name");
    				return String.format("[%s] %s",
    						element.getNodeName(),
    						(name==null)||("".equals(name))?"":name);
    			}
    		};	    	
	    	currentNode.add(newNode);
	    	buildTree(newElement,newNode);
	    	((DefaultTreeModel) tree.getModel()).nodeStructureChanged(currentNode);
    	}
    }
    
    protected Element copyElement(Element oldElement) {
    	Element newElement = doc.createElement(oldElement.getTagName());
    	NamedNodeMap attr = oldElement.getAttributes();
    	for (int i=0; i <  attr.getLength(); i++) 
    		newElement.setAttribute(attr.item(i).getNodeName(), attr.item(i).getNodeValue());
    	
    	NodeList children = oldElement.getChildNodes();
    	for (int i=0; i < children.getLength();i++)
    		if (children.item(i).getNodeType()==Node.ELEMENT_NODE) {
    			newElement.appendChild(copyElement((Element)children.item(i)));
    		}	
    	return newElement;
    }

    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();

        if (node == null) return;

        Object nodeInfo = node.getUserObject();
        if (node.isLeaf()) {
           displayElement((Node)nodeInfo);
           currentNode = null;
        } else {
           displayElement(null); 
           currentNode = node;
        }
        if (DEBUG) {
            System.out.println(nodeInfo.toString());
        }
    }
    
    private void displayElement(Node element) {
    	attributeEditor.setNode(element);
    }
	
    
    private void buildTree(Element element, DefaultMutableTreeNode node) {
    	
    	NamedNodeMap attr = element.getAttributes();
    	for (int i=0; i <  attr.getLength(); i++) {
    		DefaultMutableTreeNode subnode = new DefaultMutableTreeNode(attr.item(i));
    		//attr.item(i).getNodeName()
    		//attr.item(i).getNodeValue()
    		node.add(subnode);
    	}
    	NodeList children = element.getChildNodes();
    	for (int i=0; i < children.getLength();i++) {
    		if (children.item(i).getNodeType()==Node.ELEMENT_NODE) {
	    		DefaultMutableTreeNode subnode = new DefaultMutableTreeNode(children.item(i)) {
	    			@Override
	    			public String toString() {
	    				Element element = (Element)getUserObject();
	    				String name = element.getAttribute("name");
	    				return String.format("[%s] %s",
	    						element.getNodeName(),
	    						(name==null)||("".equals(name))?"":name);
	    			}
	    		};
	    		node.add(subnode);
	    		buildTree((Element) children.item(i), subnode);
    		}
    		/*
    		else if (children.item(i).getNodeType()==Node.ATTRIBUTE_NODE) {
    			if (children.item(i).getNodeName().equals("name")) continue;
	    		DefaultMutableTreeNode subnode = new DefaultMutableTreeNode(children.item(i));
	    		node.add(subnode);
    		}
    		*/
    	}
    	
    }

    private static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("ToxMatch config");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Add content to the window.
        frame.add(new ConfigTree());

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }
    
    public static void main(String[] args) {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }

    public void save(File file) {
    	try {
			DOMSource source = new DOMSource(doc);
			Transformer xformer = TransformerFactory.newInstance().newTransformer();
	        xformer.setOutputProperty(OutputKeys.INDENT,"Yes");
	        xformer.setOutputProperty(OutputKeys.STANDALONE,"Yes");
	
	        Result result = new StreamResult(file);
	        xformer.transform(source, result);
	        
    	} catch (Exception x) {
    		
    	}
    }
}

class AttributeEditor extends JPanel implements FocusListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1595766339909029363L;
	protected JLabel nameField;
	protected JTextField valueField;
	protected JButton button;
	protected Node element; 
	public String getName() {
		return nameField.getText();
	}
	public void setName(String name) {
		nameField.setText(name);
	}
	public String getValue() {
		return valueField.getText();
	}
	public void setValue(String value) {
		valueField.setText(value);
	}

	public void setNode(Node element) {
		this.element = element;
		setName(element==null?"":element.getNodeName());
		setValue(element==null?"":element.getNodeValue());
		button.setEnabled(element!=null);
	}
	public AttributeEditor() {
		super();
		nameField = new JLabel();
		nameField.setPreferredSize(new Dimension(300,24));
		nameField.setMaximumSize(new Dimension(300,24));
		valueField = new JTextField();
		valueField.setPreferredSize(new Dimension(300,24));
		valueField.setMaximumSize(new Dimension(300,24));		
//		nameField.setBorder(BorderFactory.createTitledBorder("Name"));
//		valueField.setBorder(BorderFactory.createTitledBorder("Value"));
		SpringLayout l = new SpringLayout();
        setLayout(l);

		add(nameField);
		add(valueField);
	
        
        button = new JButton(new AbstractAction("Update") {
        	public void actionPerformed(ActionEvent e) {
        		if (element!=null)
        		element.setNodeValue(valueField.getText());
        	}
        });
        add(button);
        button.setEnabled(element!=null);
        SpringUtilities.makeCompactGrid(this,
                3,1, //rows, cols
                2, 2,        //initX, initY
                2, 2);       //xPad, yPad	
	}
	public void focusGained(FocusEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void focusLost(FocusEvent e) {
		//if (e.getSource()==valueField)
		//	element.setNodeValue(valueField.getText());
		
	}
}
