/* VisualizationPanel.java
 * Author: Nina Jeliazkova
 * Date: Aug 10, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.awt.GridLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import ambit.ui.UITools;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.data.SelectedItem;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

/**
 * Contains {@link MyChart}, {@link SimilarityMatrixPanel}, {@link SimilarityResultViewer}.
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Aug 10, 2006
 */
public class VisualizationPanel extends JSplitPane implements PropertyChangeListener {
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 893277166119191993L;
    protected ListPanel[] subsetsPanel;
    protected MyChart chartPanel = null;
    protected SimilarityResultViewer[] similarityViewer = null;
    protected SimilarityMatrixPanel similarityMatrixViewer = null;
    protected HyperGraphSimilarityPanel hypergraph = null;
    protected static String[] pages = new String[] {"Chart","Similarity"};
    
    public VisualizationPanel(ChemCASTModule chemCastData) {
        super(JSplitPane.VERTICAL_SPLIT);

        addWidgets(chemCastData);
        
        
        for (int i=0; i < ChemCASTModule.DATA_COUNT;i++) {
        	/*
        	chemCastData.getDataset(i).addPropertyChangeListener(subsetsPanel[i]);
        	chemCastData.getDataset(i).addPropertyChangeListener(chartPanel);
        	*/
        	chemCastData.getDataset(i).addPropertyChangeListener(subsetsPanel[i]);
        	chemCastData.getDataset(i).setResultListener(similarityViewer[i]);
        	chemCastData.getDataset(i).addPropertyChangeListener(similarityMatrixViewer);        	
        }	
        
        chemCastData.addPropertyChangeListener(this);
   
        
        
       
        /*
        results_panel.getChartPanel().addPropertyChangeListe          ner(trainPanel);
        results_panel.addPropertyChangeListener(
        		ChemCASTModule.datasetName[ChemCASTModule.DATA_TRAINING]
        		,trainPanel);
*/
    }
    protected void addWidgets(final ChemCASTModule data) {
    	JPanel histogramPanel = new JPanel(new GridLayout(ChemCASTModule.DATA_COUNT,1));
    	subsetsPanel = new ListPanel[ChemCASTModule.DATA_COUNT];
    	similarityViewer = new SimilarityResultViewer[ChemCASTModule.DATA_COUNT];
    	for (int i=0; i < ChemCASTModule.DATA_COUNT; i++) {
    		similarityViewer[i] = new SimilarityResultViewer(ChemCASTModule.datasetName[i],data.getDataset(i));
    		histogramPanel.add(similarityViewer[i]);
    		subsetsPanel[i] = new ListPanel(data.getDataset(i).getType(),data.getDataset(i)) {
    			@Override
    			public void clickRow(JTable table, int row) {
        			Subsets subsets = (Subsets)((ChemCastListTableModel)table.getModel()).getList();
        			selectRow(table,row,subsets,subsets.getType());
    			}
    		};

    	}	
       
        chartPanel = new MyChart(data.getDatasets(),"","","","","","");
        similarityMatrixViewer = new SimilarityMatrixPanel();
        
        JTabbedPane tabbedPane = new JTabbedPane();
        
        tabbedPane.addTab("Chart",UITools.createImageIcon("ambit/ui/images/chart.png"),chartPanel,"Scatter plot");
        tabbedPane.addTab("Similarity histogram",UITools.createImageIcon("ambit/ui/images/histogram.png"),histogramPanel,"Similarity histogram");
        tabbedPane.addTab("Similarity matrix",UITools.createImageIcon("ambit/ui/images/similarity_matrix.png"),similarityMatrixViewer,"Similarity matrix");
//                UITools.createImageIcon("ambit/ui/images/structure_32x32.png")
        //tabbedPane.addTab("Graph",null,hypergraph,"Graph");
        
        JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
        		subsetsPanel[0],subsetsPanel[1]);
        
        /*
        add(sp,BorderLayout.NORTH);
        add(tabbedPane,BorderLayout.CENTER);
        */
        setLeftComponent(sp);
        setRightComponent(tabbedPane);

    }
    
    public void propertyChange(PropertyChangeEvent evt) {
        if (StatusBar.Message_Processing.equals(evt.getPropertyName())) {
            boolean enable = !((Boolean)evt.getNewValue()).booleanValue();
            chartPanel.setEnabled(enable);
            similarityMatrixViewer.setEnabled(enable);
            for (int i=0; i < subsetsPanel.length;i++)
            	subsetsPanel[i].setEnabled(enable);
        } else {
        	//System.out.println(getClass().getName() + " " + evt.getPropertyName());
        	/*
        	if (evt.getPropertyName().equals(Subsets.p_groups)) 
        		subsetsPanel.setList(evt.getNewValue().toString(),(Subsets) evt.getNewValue());
        		*/
            //chartPanel.propertyChange(evt);
        	
        	/*
        	for (int i=0; i < similarityViewer.length;i++)
        		similarityViewer[i].propertyChange(evt);
        		*/
            similarityMatrixViewer.propertyChange(evt);
            //hypergraph.propertyChange(evt);
        }
    }
    protected void errorMessage(Exception x) {
        JOptionPane.showMessageDialog(this,x.toString());        
    }
	public void selectRow(JTable table, int row, Subsets data,String propertyName) {
		Subset subset = (Subset) table.getValueAt(row,1);
		//System.out.println(subset);
		try {
            SelectedItem newSelected = new SelectedItem(
        			data,
        			subset,
                    0);
            data.firePropertyChange(propertyName, 
                null, 
                newSelected
                );
		} catch (Exception x) {
            errorMessage(x);
		}
		
	}

}
