/* TrainingSetOpenAction.java
 * Author: Nina Jeliazkova
 * Date: Oct 1, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui.actions;

import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.swing.Icon;
import javax.swing.JFrame;

import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import ambit.io.FileInputState;
import ambit.io.MolPropertiesIOSetting;
import ambit.io.MyIOUtilities;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Endpoints;
import chemCast.similarity.data.Subsets;
import chemCast.ui.wizard.EndpointsWizard;

public class ConfiguredTrainingSetOpenAction extends FileOpenAction {

    /**
     * 
     */
    private static final long serialVersionUID = -8512684041542159860L;
    protected boolean showFileOptions = false;
    public ConfiguredTrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            int dataset) {
        this(userData, mainFrame, "Predefined training sets", dataset);
    }

    public ConfiguredTrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, int dataset) {
        this(userData, mainFrame, name, icon, dataset);
    }

    public ConfiguredTrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, Icon icon, int dataset) {
        super(userData, mainFrame, name, icon, dataset);
        putValue(SHORT_DESCRIPTION,"Opens training data sets for preconfigured endpoints");
    }
    public DatasetDef getDataset() throws SubsetException {
    	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading endpoints");
        Endpoints endpoints = ChemCASTModule.readEndpoints();
        int index = endpoints.indexOf(((ChemCASTModule)userData).getEndpoint());
        subsets.clear();
        List<Subsets> l = new ArrayList<Subsets>();
        l.add(subsets);
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Selecting endpoints");
        EndpointsWizard wizard = new EndpointsWizard(mainFrame,endpoints,l,index);
        wizard.getDialog().setTitle("Select an endpoint");

        if (wizard.showModalDialog() == 0) {
            ((ChemCASTModule) userData).setEndpoint(wizard.getEndpoint());
            //((ChemCASTModule) userData).setSubsets(wizard.getSubset());
            subsets = wizard.getSubset();
            //File file = new File(wizard.getEndpoint().getSource().getDatasetValue());
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,wizard.getEndpoint().getTrainingData()+" selected");
            wizard.getEndpoint().getTrainingData().setColor(Color.blue);
            DatasetDef dataset = wizard.getEndpoint().getTrainingData();
            showFileOptions = "New".equals(wizard.getEndpoint().getName());
            if (dataset.getDatasetSource().equals(DatasetDef.p_filesource)) {
                File file = new File(dataset.getDatasetValue());
                if (!file.exists()) {
                    file = MyIOUtilities.selectFile(mainFrame,null,
                            getChemCastData().getDefaultDir(),
                            FileInputState.extensions,FileInputState.extensionDescription,true);
                    if (file != null) 
                        dataset.setDatasetValue(file.getAbsolutePath());
                    else return null;
                }    
            }
            return wizard.getEndpoint().getTrainingData();
            
        } else {
        	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null," ");
        	return null;
        }
        /*
        System.out.println("Dialog return code is (0=Finish,1=Cancel,2=Error): " + ret);
        System.out.println(wizard.getEndpoint());
        Subsets sets = wizard.getSubset();
        System.out.println(sets);
        List<? extends IChemCastObject> list = sets.getList();
        for (int i=0; i < list.size();i++)
            if (list.get(i).isEnabled())
                System.out.println(list.get(i));
                */
    }
    @Override
    protected IReaderListener getReaderListener() {
        if (!showFileOptions) return null;

        return new IReaderListener() {
          public void frameRead(ReaderEvent e) {
              getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
              //getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading record");
          }
          public void processIOSettingQuestion(IOSetting setting) {
              if (setting instanceof MolPropertiesIOSetting) {
                  dataset.setProperties(((MolPropertiesIOSetting)setting).getProperties());
                  System.out.println(dataset.getProperties());
                  
              }  
          }
        };
    }    

}
