/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import weka.core.Instance;
import weka.core.Instances;

/**
 * Euclidean distance Dab=Math.sqrt(Zaa+Zbb-2*Zab)
 * @author Nina Jeliazkova
 *
 */
public class EuclideanSimilarity extends AbstractSimilarity {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5454268886009582012L;
	public EuclideanSimilarity() {
		super();
	}

	public EuclideanSimilarity(Instances data) {
		super(data);

	}

    public float getDistance(Instance a, Instance b) throws Exception {
        return (float) distance(a, b);
    }

    public float getNativeComparison(Instance a, Instance b) throws Exception {
        return (float) distance(a, b);
    }    
	@Override
	public double distance(Instance a, Instance b) throws Exception {
		return Math.sqrt(distance(a,b,Double.MAX_VALUE,false));
	}
	@Override
	public double distance(Instance a, Instance b, double arg2) {
		return distance(a, b, arg2, false);
	}
	@Override
	public double distance(Instance a, Instance b, double arg2, boolean print) {
		return zAA(a)+zAA(b)-2*zAB(a,b);
	}	
	@Override
	public void postProcessDistances(double[] distances) {
	    for(int i=0; i<distances.length; i++) 
	        distances[i] = Math.sqrt(distances[i]);

	}

}


