/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.smarts;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import joelib.molecule.JOEAtom;
import joelib.molecule.JOEMol;
import joelib.smarts.JOESmartsPattern;
import joelib.smiles.JOESmilesParser;
import junit.framework.TestCase;

import org.openscience.cdk.Atom;
import org.openscience.cdk.Molecule;
import org.openscience.cdk.exception.NoSuchAtomException;
import org.openscience.cdk.isomorphism.IsomorphismTester;
import org.openscience.cdk.tools.LoggingTool;

import toxTree.exceptions.DecisionMethodException;
import toxTree.query.FunctionalGroups;
import toxTree.tree.rules.RuleAllSubstructures;
import toxTree.tree.rules.smarts.Convertor;
import toxTree.tree.rules.smarts.RuleSMARTSubstructure;
import toxTree.tree.rules.smarts.SmartsPattern;



public class RuleSMARTSubstructureTest extends TestCase {
	
	public static LoggingTool logger = new LoggingTool(RuleAllSubstructures.class);
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(RuleSMARTSubstructureTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for RuleAnySubstructureTest.
	 * @param arg0
	 */
	public RuleSMARTSubstructureTest(String arg0) {
		super(arg0);
		
	}
	public void testRuleSMARTSubstructure() {
		RuleSMARTSubstructure rule = new RuleSMARTSubstructure();		
//		Molecule mol = MoleculeFactory.makeBenzene();
		
        Molecule mol = new Molecule();
        mol.addAtom(new Atom("C")); // 0
        mol.addAtom(new Atom("C")); // 1
        mol.addAtom(new Atom("C")); // 2
        mol.addAtom(new Atom("C")); // 3
        mol.addAtom(new Atom("C")); // 4
        mol.addAtom(new Atom("C")); // 5

        mol.addBond(0, 1, 1); // 1
        mol.addBond(1, 2, 2); // 2
        mol.addBond(2, 3, 1); // 3
        mol.addBond(3, 4, 2); // 4
        mol.addBond(4, 5, 1); // 5
        mol.addBond(5, 0, 2); // 6

        
		String  smarts = ("c1ccccc1");		
		String  smarts1 = ("CCCCCC");
		try {
			rule.setContainsAllSubstructures(false);
			rule.addSubstructure(smarts);
			rule.addSubstructure(smarts1);
//			HueckelAromaticityDetector.detectAromaticity(mol);
			//assertTrue(rule.verifyRule(mol,smarts));
			//assertFalse(rule.verifyRule(mol,smarts1));
			assertTrue(rule.verifyRule(mol));
			//assertFalse(rule.verifyRule(mol));
			
		} catch (toxTree.exceptions.DecisionMethodException x) {
			fail();

		}
		
	}
    public void testCDKJoeMolMatch() {
        Molecule mol = new Molecule();
        
        mol.addAtom(new Atom("C")); // 0
        mol.addAtom(new Atom("C")); // 1
        mol.addAtom(new Atom("C")); // 2
        mol.addAtom(new Atom("C")); // 3
        mol.addAtom(new Atom("C")); // 4
        mol.addAtom(new Atom("O")); // 5

        mol.addBond(0, 1, 1); // 1
        mol.addBond(1, 2, 1); // 2
        mol.addBond(2, 3, 1); // 3
        mol.addBond(3, 4, 1); // 4
        mol.addBond(4, 5, 1); // 5

	
        JOEMol converted = Convertor.convert(mol);
        
        JOEMol joemol=new JOEMol();
    	String smiles="CCCCCO";
    	if (!JOESmilesParser.smiToMol(joemol, smiles,""))
    	 {
    	    System.err.println("SMILES entry \"" + smiles + "\" could not be loaded.");
    	 }
    	 assertEquals(joemol.toString(),converted.toString());
    	

        JOESmartsPattern smartPatern = new JOESmartsPattern();
		if(!smartPatern.init("CCCCO")){				   
			System.err.println("Invalid SMARTS pattern.");
		}		
		
		assertTrue(smartPatern.match(converted));
		

    }
    public void testCDKJoeMol() {
        Molecule mol = new Molecule();
        
        mol.addAtom(new Atom("C")); // 0
        mol.addAtom(new Atom("O")); // 5

        mol.addBond(0, 1, 1); // 1

	
        JOEMol converted = Convertor.convert(mol);
        Molecule reverted = Convertor.convert(converted);

        assertEquals(mol.getAtomCount(), reverted.getAtomCount());
        assertEquals(mol.getBondCount(), reverted.getBondCount());
        
        try {
            IsomorphismTester it = new IsomorphismTester(mol);
            assertTrue(it.isIsomorphic(reverted));
        } catch (NoSuchAtomException e) {
        	e.printStackTrace();
            assertTrue(false);
        }
    }
    public void testAtom() {
        Atom a = new Atom("C");
        a.setX3d(1.0);
        a.setY3d(2.0);
        a.setZ3d(3.0);

        JOEAtom converted = Convertor.convert(a);
        Atom reverted = Convertor.convert(converted);

        assertEquals(a.getAtomTypeName(), reverted.getAtomTypeName());
        /*
        assertTrue(a.getPoint3d().x == reverted.getPoint3d().x);
        assertTrue(a.getPoint3d().y == reverted.getPoint3d().y);
        assertTrue(a.getPoint3d().z == reverted.getPoint3d().z);
        */
    }
    
    public void testCDKJoeMolAllC() {
        Molecule mol = new Molecule();
        
        mol.addAtom(new Atom("C")); // 0
        mol.addAtom(new Atom("C")); // 5

        mol.addBond(0, 1, 1); // 1

	
        JOEMol converted = Convertor.convert(mol);
        Molecule reverted = Convertor.convert(converted);

        assertEquals(mol.getAtomCount(), reverted.getAtomCount());
        assertEquals(mol.getBondCount(), reverted.getBondCount());
        
        try {
            IsomorphismTester it = new IsomorphismTester(mol);
            assertTrue(it.isIsomorphic(reverted));
        } catch (NoSuchAtomException e) {
            assertTrue(false);
        }
    }    
    /**
     * Creates {@link JOEMol} and tests if {@link  JOESmartsPattern} works.
     *
     */
    /*public void testDirectJoeMolMatch() {
    	JOEMol mol=new JOEMol();
    	RuleSMARTSubstructure rule = new RuleSMARTSubstructure();
    	String smiles="c1ccccc1";
    	if (!JOESmilesParser.smiToMol(mol, smiles,"mol"))
    	 {
    	    System.err.println("SMILES entry \"" + smiles + "\" could not be loaded.");
    	 }
    	 //System.out.println(mol.toString());
    	 /*JOESmartsPattern smartPatern = new JOESmartsPattern();
 		if(!smartPatern.init("c1ccccc1")){				   
 			System.err.println("Invalid SMARTS pattern.");
 		}		
 		
 		assertTrue(smartPatern.match(mol));
    	Enumeration e  = rule.smartsPatterns.keys();
    	while(e.hasMoreElements()){
    		rule.hasSMARTSPattern(mol, e.nextElement().toString());
    	}
    }*/
    public void testDirectJoeMolMatch() {
    	String[][] testSmarts = {
    			//name,smarts,smiles
    			{"Hydroperoxide","[OX2H][OX2]","OO"},
    			{"Aldehyde","[$([CX3H][#6]),$([CX3H2])]=[OX1]","CC=O"},
    			{"Epoxide","[OX2r3]1[#6r3][#6r3]1","C1OC1C"},
    			{"Ketene","[CX3]=[CX2]=[OX1]","CC=C=O"},
    			{"Ketone","[#6][CX3](=[OX1])[#6]","C=CC(=O)C"},
    			{"Alkene","[CX3;$([H2]),$([H1][#6]),$(C([#6])[#6])]=[CX3;$([H2]),$([H1][#6]),$(C([#6])[#6])]","C=C"},
    			{"Thioamide","[$([CX3;!R][#6]),$([CX3H;!R])](=[SX1])[#7X3;$([H2]),$([H1][#6;!$(C=[O,N,S])]),$([#7]([#6;!$(C=[O,N,S])])[#6;!$(C=[O,N,S])])]","CCOC(=O)N1C=CN(C)C1(=S)"}
    			
    	};
    	
    	for (int i=0; i < testSmarts.length;i++) {
    		RuleSMARTSubstructure rule = new RuleSMARTSubstructure();
    		System.out.print(testSmarts[i][0]);
    		try {
    			rule.addSubstructure(testSmarts[i][1]);
    			assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer(testSmarts[i][2],true)));
    			System.out.println("\tOK");	
    		} catch (DecisionMethodException x) {
    			System.out.println(x.getMessage());
    			fail();
    		}
    	}
    	
    	/*
    	try{
    		String Thioamide = "[$([CX3;!R][#6]),$([CX3H;!R])](=[SX1])[#7X3;$([H2]),$([H1][#6;!$(C=[O,N,S])]),$([#7]([#6;!$(C=[O,N,S])])[#6;!$(C=[O,N,S])])]";
        	//String Hydroperoxide = "[OX2H][OX2]";
        	//String Aldehyde = "[$([CX3H][#6]),$([CX3H2])]=[OX1]";
        	//String Ketene = "[CX3]=[CX2]=[OX1]";
        	//String Epoxide = "[OX2r3]1[#6r3][#6r3]1";
        	//String Ketone = "[#6][CX3](=[OX1])[#6]";
        	//String Alkene = "[CX3;$([H2]),$([H1][#6]),$(C([#6])[#6])]=[CX3;$([H2]),$([H1][#6]),$(C([#6])[#6])]";
        	//rule.initSingleSMARTS(rule.smartsPatterns,"1", Alkene); 
    		
    		rule.addSubstructure(Thioamide);
        	//rule.initSingleSMARTS(rule.getSmartsPatterns(),"1", Thioamide);
        	//rule.initSingleSMARTS(rule.smartsPatterns,"1", Hydroperoxide);
        	//rule.initSingleSMARTS(rule.smartsPatterns,"1", Aldehide);
        	//rule.initSingleSMARTS(rule.smartsPatterns,"1", Ketene);
        	//rule.initSingleSMARTS(rule.smartsPatterns,"1", Epoxide);
        	  //rule.initSingleSMARTS(rule.smartsPatterns,"1", Ketone);
    		//Carbimazole
    		assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("CCOC(=O)N1C=CN(C)C1(=S)",true)));
    		//Ethylene
    		//assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("C=C",true)));
    		//Hydrogen peroxide
 		    //assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("OO",true)));
    		//Ethanal(Aldehide)
    		//assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("CC=O",true)));
    		//Methyl ketene  
    		//assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("CC=C=O",false)));
    		//Propylene oxide
    		//assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("C1OC1C",false)));
    		//Methyl vinyl ketone  
    		  //assertTrue(rule.verifyRule(FunctionalGroups.createAtomContainer("C=CC(=O)C",false)));
    	} 
    	catch (SMARTSException x) {
    		fail();
    	}
    	catch (toxTree.exceptions.DecisionMethodException x) {
    		fail();		
    	}
    	*/
    }
    public void testXMLSerializingSmartsPatterns() {
    	try {
    		SmartsPattern rule = new SmartsPattern("[OX2H][OX2]");
			FileOutputStream os = new FileOutputStream("smarts.xml");
			XMLEncoder encoder = new XMLEncoder(os);
			encoder.writeObject(rule);
			encoder.close();	
			
			FileInputStream in = new FileInputStream("smarts.xml");
			XMLDecoder decoder = new XMLDecoder(in);
			SmartsPattern rule1 = (SmartsPattern)decoder.readObject();
			assertEquals("[OX2H][OX2]",rule1.getSmarts());
			decoder.close();
	
		} catch (Exception x) {
			x.printStackTrace();
			fail();
		}
    }
    
    public void testXMLSerializing() {
    	try {
    		RuleSMARTSubstructure rule = new RuleSMARTSubstructure();
    		String smarts = "[OX2H][OX2]";
   			rule.addSubstructure("1",smarts);
			FileOutputStream os = new FileOutputStream("rulesmarts.xml");
			XMLEncoder encoder = new XMLEncoder(os);
			encoder.writeObject(rule);
			encoder.close();	
			
			FileInputStream in = new FileInputStream("rulesmarts.xml");
			XMLDecoder decoder = new XMLDecoder(in);
			RuleSMARTSubstructure rule1 = (RuleSMARTSubstructure)decoder.readObject();
			assertEquals(1,rule1.getSmartsPatterns().size());
			assertEquals(smarts,rule1.getSmartsPatterns().get("1").toString());
			decoder.close();
	
		} catch (Exception x) {
			x.printStackTrace();
			fail();	
		}
    }
    
    
}
